/*********************************************************************/
/*                                                                   */
/* EXEC NAME       : FKXECMON  <=== PROTOTYPE ===>                   */
/*                                                                   */
/* DESCRIPTIVE NAME: Routine for Connection Monitoring & Thesholding */
/*                                                                   */
/* FUNCTION        : Monitor IP connections, comparing actual data   */
/*                   from the "ibmtcpipmvstcpconntable" MIB table    */
/*                   against policy specifications in the IPCONN     */
/*                   policy specifications, taking automated         */
/*                   action(s) as defined in the policy.             */
/*                                                                   */
/*                                                                   */
/* NOTES           : Customer modifiable variables are located in    */
/*                   CUSTOMER_VARS                                   */
/*                                                                   */
/* Return Codes    : RC =  0 All is well!                            */
/*                   RC =  3 IPADDR &/or HOSTNAME missing            */
/*                   RC =  4 Missing or invalid parms                */
/*                   RC =  9 Security authorization failure          */
/*                           (See FKXESCMD for further details)      */
/*                   RC = 10 No MIB Data                             */
/*                   All others = See FKXESCMD for further details   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/* DEPENDENCIES    : NetView PIPEs                                   */
/*                   SNMP V1 or V2 w/ SET authority (to break conn)  */
/*                                                                   */
/* MESSAGES        : FKX101 CONN IDLE - NOTIFY                       */
/*                   FKX102 CONN IDLE - DROP                         */
/*                   FKX104 MIN BYTES THRESH - NOTIFY                */
/*                   FKX105 MIN BYTES THRESH - DROP                  */
/*                   FKX107 MAX BYTES THRESH - NOTIFY                */
/*                   FKX108 MAX BYTES THRESH - DROP                  */
/*                                                                   */
/* OPERATION       : Same as function above                          */
/*                                                                   */
/* GLOBALS         :   EZLTRACED - Domain Trace Global               */
/*                 :   EZLTRACEO.opid - Operator Trace Global        */
/*                 :   CNMSTYLE.CONNMONPORTS                         */
/*                 :   CNMSTYLE.CONNMONFILE                          */
/*                 :   CNMSTYLE.CONNMONINTVL                         */
/*                                                                   */
/*                                                                   */
/* CHANGE ACTIVITY :                                                 */
/* APAR#    DATE       ID DESCRIPTION                                */
/* -------- --------   -- -----------------------------------------  */
/* $DQ=Dxxxxxxx,TIVNV52,09/05/06,PJQ: Updated to use TCPCONN instead */
/*                                    of SNMP WALK.                  */
/*                                                                   */
/*                                                                   */
/*-------------------------------------------------------------------*/
 Trace Off
 parse source . Invoc Ident .
 parse arg argstring                                 /* set argstring*/
 Date = date('U')                                    /* mm/dd/yy fmt */
 Time = substr(time(),1,5)                           /* hh:mm fmt    */
 /* Bypass Security since this is "internal" processing              */
 Call AUTBYPAS('ON')

/*-------------------------------------------------------------------*/
/* Entry Trace Call                                                  */
/*-------------------------------------------------------------------*/
 'GLOBALV GETC EZLTRACED EZLTRACEO.'Opid()   /* Trace Variable       */

If Substr(EZLTRACED,1,4) <> 'NONE' Then  /* If not NONE ...          */
  Do
   'EZLTRACE ENTRY 'ident argstring                  /* Entry trace  */
   interpret 'trace' EZLTRACER
  End
OperTrace = Substr(Value('EZLTRACEO.'Opid()),1,2)

signal on NOVALUE                                    /* RAS signals  */
signal on SYNTAX
signal on HALT
signal on FAILURE

ADDRESS NETVASIS

/*-------------------------------------------------------------------*/
/* Main Program                                                      */
/* Make sure you set: Tower, Resource_name, Resource_type, Badcmd    */
/*                    Errmsg BadParm BadParmVal Return_code          */
/*-------------------------------------------------------------------*/

/* Initialize all local variables, including cusomer modifiable ones.*/
Call Init
Call Customer_Vars

/* Parse input parms & validate CNMSTYLE common globals.             */
Call Parse_Validate_Input

Call Get_Policy                       /* retrieve IPCONN policy defs */

/* Retrieve Community Name, IP Addr, & HostName for each IP stack.   */
If Go_On = 'Y' then
  Call Get_Stack_Info

/* Retrieve port data from file or MIBs based on CONNMONPORTS global.*/
If Go_On = 'Y' then
  Call Get_Port_Data

/* Use port data, IPCONN policy, TCPCONN data, & MIB data to perform */
/* monitoring and thresholding functions.                            */
If Go_On = 'Y' then
  Call Process_Policy

/* Always reschedule this timer, even if no data, security error, etc*/
/* Timer interval is defined in CNMSTYLE.CONNMONINTVL                */
Call Reschedule_Timer

Call Leave_now                                       /*tidy exit     */


Customer_Vars:
/*-------------------------------------------------------------------*/
/* Customer_Vars:                        = = = >   L 0 0 K   < = = = */
/* Customers can modify these variables.                             */
/*-------------------------------------------------------------------*/

/* Define a default SNMP community name in case it it not defined on */
/* a stack policy definiton (TCP390). The preferred method is to use */
/* the COMMUNITYNAME keyword on your TCP390 definition.              */
Default_Cname    = 'publicv2c'

/* Define a default file name to contain port data. The preferred    */
/* method is to define the file name in CNMSTYLE with:               */
/* COMMON.CNMSTYLE.CONNMONFILE = CONNMON                             */
Default_filename = 'CONNMON'

/* Define a default time interval used to schedule this routine.     */
/* The preferred method to define the interval is in CNMSTYLE with:  */
/* COMMON.CNMSTYLE.CONNMONINTVL = 00:10:00                           */
Default_interval = '00:10:00'

/* Define a default value for the MAXRECS parm of the TCPCONN cmd.   */
/* The preferred method to define MAXRECS on the IPCONN policy       */
/* definition.                                                       */
Default_MaxRecs = 100

/* Define a default value for the USEIPADDR keyword of the IPCONN    */
/* The preferred method to define USEIPADDR on the IPCONN policy     */
/* policy definition.                                                */
/* definition.                                                       */
Default_UseIpAddr = 'NO'

Return



Init:
/*-------------------------------------------------------------------*/
/* Init:                                                             */
/* Initialize all local clist vars                                   */
/*-------------------------------------------------------------------*/

Go_On = 'Y'
Errormsg = ''
ExitMsg = ''
Tower = 'AON'
Return_Code = 0

sp = ''
Default_Stack = ''
StackList = ''
MaxRecs = ''
UseIpAddr = ''
UpdType = ''
PortIntvl = ''
Notify_sp = ''
interval = ''
First_Time = 0
BadParm = ''
Port_Array.0 = 0
PortData.0 = 0

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* define loopback & inaddrany vars.  Used to NLOC in PIPE data      */
LoopBack = '.127.0.0.1'               /* loopback interface addr  (1)*/
InAddrAny = '.0.0.0.0'                /* InAddrAny specification  (2)*/
BoundAddr = '.0.0.0.0.0'              /* BoundAddr specification  (3)*/
/* we will use these 3 vars to eliminate the following data:         */
/* ibmMvsTcpConnResourceName.127.0.0.1.1025.0.0.0.0.0 = TCPIP     (1)*/
/* ibmMvsTcpConnResourceName.0.0.0.0.21.0.0.0.0.0 = FTPD1         (2)*/
/* ibmMvsTcpConnResourceName.9.67.50.52.9080.0.0.0.0.0 = T510EENV (3)*/
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/


sep = '|'                             /* PIPE stage separator char   */
endit = '?'                           /* PIPE END stage character    */


/*- - - - - - - - - - - MIB  Definitions  - - - - - - - - - - - - - -*/
/* ** define the MIB variables in OID format:                     ** */
ibmTcpipMvsTcpConnTable = '.1.3.6.1.4.1.2.6.19.2.2.7'
/* ***************************************************************** */
/* ** define the MIB vars used in OID format:                     ** */
ibmMvsTcpConnLastActivity = '.1.3.6.1.4.1.2.6.19.2.2.7.1.1.1.'
ibmMvsTcpConnBytesIn = '.1.3.6.1.4.1.2.6.19.2.2.7.1.1.2.'
ibmMvsTcpConnBytesOut = '.1.3.6.1.4.1.2.6.19.2.2.7.1.1.3.'
ibmMvsTcpConnSndWnd = '.1.3.6.1.4.1.2.6.19.2.2.7.1.1.34.'
ibmMvsTcpConnResourceName = '.1.3.6.1.4.1.2.6.19.2.2.7.1.1.37.'
ibmMvsTcpConnResourceId = '.1.3.6.1.4.1.2.6.19.2.2.7.1.1.39.'

/* ** tcpConnState is set to break the connection (action=DROP)   ** */
tcpConnState = '.1.3.6.1.2.1.6.13.1.1'
mibtype = 'i'                         /* for SNMP SET, type integer  */
/* ***************************************************************** */

TcpConn_Cmd = 'TCPCONN QUERY SELECT=ACT MAXRECS='

Snmp_GET = 'snmp get -v2c -Obestn'

Snmp_SET = 'snmp SET -v2c -Obestn'

mib_cmd = ''

'GLOBALV GETC CNMSTYLE.CONNMONPORTS CNMSTYLE.CONNMONFILE',
  ' CNMSTYLE.CONNMONINTVL EZLNLOG'
interval = CNMSTYLE.CONNMONINTVL
If interval = '' then
  interval = Default_interval
Else
  Call Validate_Interval

/* Check if AON is enabled - with AON comes the NOTIFY policy.      */
/* Without AON we must issue REXX SAY of message to get to opers.   */
AON_Enabled = 'Y'
If Tower('AON') = 0 then                /* AON is not enabled       */
  Do
    AON_Enabled = 'N'
    /* If EZLNLOG is null set it to N to prevent SLN errors w/o AON*/
    If Symbol('EZLNLOG') = 'LIT' | EZLNLOG = '' then
      Do
        EZLNLOG = 'N'
        'GLOBALV PUTC EZLNLOG'
      End
  End


Return



Parse_Validate_Input:
/*-------------------------------------------------------------------*/
/* Parse_Validate_Input:                                             */
/* Parse input parameters and validate them.                         */
/*-------------------------------------------------------------------*/

Parse Var Argstring Default_Stack .

If Default_Stack = '' then              /* no default stack?         */
  Do                                    /* Determine local stack     */
    'PIPE NETV POLICY ENTRY=TCP390 ',
    sep ' CORR ',
    sep ' SEP ',
    sep ' LOC /DOMAIN LOCAL/ ' ,
    sep ' VAR local_stack '
    /* response: EZL115I  TCP390 TIVED1 DOMAIN LOCAL                 */
    If Symbol('local_stack') = 'VAR' then
      parse var local_stack . Default_Stack .
  End                                   /* Determine local stack     */



/* Test file contents. 1 line means this is first time being called  */
filename = Value(CNMSTYLE.CONNMONFILE)
If filename = '' then
  filename = Default_filename
'PIPE < DSIPARM.'filename ,
  sep ' Stem PortData.'
If PortData.0 = 1 then              /*  initial call ...             */
  First_Time = 1

/* Retrieve port data from file (static or semi-dynamic) or MIBs if  */
/* data is to be collected dynamically each time.                    */
Parse var CNMSTYLE.ConnMonPorts '('UpdType','PortIntvl')'
UpdType = Strip(UpdType,'T',')')
Select                              /* determine type of processing  */
  When UpdType = 'STATIC' then      /* port data is STATIC           */
    Nop
  When UpdType = 'DYNAMIC' &,
    PortIntvl <> '' then            /* port data is semi-DYNAMIC     */
      UpdType = 'SEMIDYNAMIC'
  When PortIntvl = '' then
      UpdType = 'DYNAMIC'
  Otherwise
End                                 /* determine type of processing  */


Return


Get_Port_Data:
/*-------------------------------------------------------------------*/
/* Get_Port_Data:                                                    */
/* Handle processing of port data retrieval.                         */
/*-------------------------------------------------------------------*/

Port_Array.0 = 0

Do i = 1 to words(StackList)        /* Loop through known stacks     */
  Curr_Stack = Word(StackList,i)
  Select                            /* get port data                 */
    When UpdType = 'DYNAMIC' then
      Do                            /* refresh port data every time  */
      'GETPORTS' Curr_Stack
      /* Retrieve port data from SAFE - contains current data        */
      'PIPE SAFE 'filename' SEIZE' ,
        sep ' Stem Port_Array. APPEND'
      End                           /* refresh port data every time  */
  When UpdType = 'SEMIDYNAMIC' then
    Do                              /* semi-Dynamic                  */
      /* If this is the initial call need to collect port data and   */
      /* store in the file. This will schedule timer to update port  */
      /* data based on the interval passed.                          */
     If First_Time = 1 then
       'GETPORTS' Curr_Stack Interval/* get port data and sched timr */
      /* File contains 'most recent' port data.                      */
     'PIPE < DSIPARM.'filename ,
        sep ' DROP FIRST ',
        sep ' Stem Port_Array. APPEND'
    End                             /* semi-Dynamic                  */
  When UpdType = 'STATIC' then      /* port data is static           */
      /* File contains snapshot of port data - collected once only.  */
     'PIPE < DSIPARM.'filename ,
        sep ' DROP FIRST ',
        sep ' Stem Port_Array.'
    Otherwise
  End                               /* get port data                 */
End                                 /* Loop through known stacks     */

If Port_Array.0 = 0 then             /* NO port data.  Error?        */
  Go_On = 'N'

Return


/*-------------------------------------------------------------------*/
/* Get_Stack_Info:                                                   */
/* Get SNMP community name, IP addr, & host name for each stack.     */
/*-------------------------------------------------------------------*/
Get_Stack_Info:
do i = 1 to Words(StackList)
  CurrentStack = Word(StackList,i)
  Call Get_Cname CurrentStack
  /* afterwards, a local var of the stack name = cname & ipaddr      */
end


Return


/*-------------------------------------------------------------------*/
/* Get_Cname:                                                        */
/* Retrieve the Community Name from the TCP390 policy definition.    */
/* Also, retrieve the IPADDR & HOSTNAME keywords to determine        */
/* where to send the snmp request(s).                                */
/*-------------------------------------------------------------------*/
Get_Cname:

Arg Sp

If value(sp) = sp then
  SpVal = ''
Else
  SpVal = Value(sp)

cname = ''
sp_ipaddr = ''
sp_hname = ''
snmp_name = ''
snmpdata = ''

If SpVal = '' then                    /* No known sp data?           */
  Do                                  /* get cname, ipaddr, etc      */
    'PIPE (STAGESEP' sep') NETV ' ,
    'EZLEPOLY REQ=GET ENTRY=TCP390 TYPE='sp ,
       sep ' CORR ',
       sep ' SEP ',
       sep ' LOC /COMMUNITYNAME / /IPADDR / /HOSTNAME / ',
       sep ' stem pipe.'

    /* TCP390 NMPIPL10 COMMUNITYNAME public */
    do j = 1 to pipe.0
      parse var pipe.j . . pipeparm pipedata
      select
        when pipeparm = 'COMMUNITYNAME' then
           cname = pipedata
        when pipeparm = 'IPADDR' then
           sp_ipaddr = pipedata
        when pipeparm = 'HOSTNAME' then
           sp_hname = pipedata
        otherwise
      end
    end

    /* set snmp hostname var to an ip addr                           */
    Select
      When sp_ipaddr <> '' then       /* got an ipaddr defined?      */
        snmp_name = sp_ipaddr
      When sp_hname <> '' then        /* hostname defined?           */
        Call Get_Host_By_Name         /* get its ipaddr then         */
      Otherwise                       /* oops - missing ipaddr/hostnm*/
        Call ERROR_MSG651
    End

    If cname = '' then
      cname = Default_cname

    /* Now, concatenate the community name & snmp name in            */
    snmpdata = "CNAME="cname" SNMPNAME="snmp_name
    spdata = spdata' 'snmpdata
    Interpret sp '= snmpdata'         /* save info under SP          */
  End                                 /* get cname, ipaddr, etc      */
Else                                  /* prior SP data saved         */
    spdata = spdata' 'Value(sp)


Return




/*-------------------------------------------------------------------*/
/* Validate_Interval:                                                */
/* Validate the interval from the ACTMON IPCONN policy def.          */
/*-------------------------------------------------------------------*/
Validate_Interval:

If Interval = '' then                 /* If still no interval ...    */
  Do                                  /* issue EZL203I message       */
    Badparm = 'INTVL'                 /*                        @P3C */
    Call ERROR_MSG203
  End                                 /* issue EZL203I message       */

parse var interval hh ':' mm ':' ss
hh = right(hh,2,'0')
mm = right(mm,2,'0')
ss = right(ss,2,'0')


Select
  When hh > 24 | hh < 00 | datatype(hh) <> 'NUM' then
    Call ERROR_MSG203
  When mm > 59 | mm < 00 | datatype(mm) <> 'NUM' then
    Call ERROR_MSG203
  When ss > 59 | ss < 00 | datatype(ss) <> 'NUM' then
    Call ERROR_MSG203
  When hh = 24 & mm > 00 then
    Call ERROR_MSG203
  Otherwise
    Badparm = ''
End

Return






/*-------------------------------------------------------------------*/
/* Get_Policy:                                                       */
/* Retrieve the IP Connection policy definitions.                    */
/*-------------------------------------------------------------------*/
Get_Policy:

cntr = 0

/* Retrieve all IPCONN policy and put into pipestem array            */
'PIPE (STAGESEP' sep') NETV ' ,
'EZLEPOLY REQ=GET ENTRY=IPCONN TYPE=*',
   sep ' CORR ',
   sep ' SEP ',
   sep ' NLOC /ACTMON /',
   sep ' stem pipestem.'

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* Commentary about what the IPCONN policy input will look like and  */
/* what the expected output will look like.                          */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Example input: (pipestem. array)                                  */
/* IPCONN TCP* SP NMPIPL10                                           */
/* IPCONN TCP* LPORT (23)                    <== may not be coded    */
/* IPCONN TCP* MAXRECS 100                         <== optional      */
/* IPCONN TCP* THRESH (*,MIN,00:04:00,NOTIFY,DROP) <== optional      */
/* IPCONN TCP* THRESH (2147,MIN,00:00:30,DROP)     <== optional      */
/* IPCONN TCP* ACTMON IPCONN                 <== removed by NLOC     */
/* IPCONN FTP  SP NMPIPL10                                           */
/* IPCONN FTP  LPORT (21)                    <== may not be coded    */
/* IPCONN FTP  THRESH (*,MIN,00:04:00,NOTIFY,DROP) <== optional      */
/* IPCONN FTP  THRESH (2147,MIN,00:00:30,DROP)     <== optional      */
/* IPCONN FTP  ACTMON IPCONN                 <== removed by NLOC     */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Example output:                                                   */
/* The end result when done processing the IPCONN policy will be a
   2 dimensional array (IPCONN.i.j) similar to:
IPCONN.1.0 = 5
IPCONN.1.1 = TCP* SP NMPIPL10
IPCONN.1.2 = TCP* LPORT (23)
IPCONN.1.3 = TCP* THRESH (*,MIN,00:04:00,NOTIFY,DROP)
IPCONN.1.4 = TCP* THRESH (2147,MIN,00:00:30,DROP)
IPCONN.1.5 = TCP* MAXRECS 100
IPCONN.1.6 = TCP* USEIPADDR NO

IPCONN.2.0 = 4
IPCONN.2.1 = FTP SP NMPIPL10
IPCONN.2.2 = FTP LPORT (21)
IPCONN.2.3 = FTP THRESH (*,MIN,00:04:00,NOTIFY,DROP)
IPCONN.2.4 = FTP THRESH (2147,MIN,00:00:30,DROP)
*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

Conn_cnt = 0
Def_cnt = 0
Tot_cnt = 0
All_cnt = 0

/* IPCONN.Conn_Cnt.Def_Cnt = IPCONN policy definition                */
prevappl = ''
spdata = Default_Stack                                       /* @PQC */
If Default_Stack <> '' then                                  /*2@PQA */
  StackList = Default_Stack
idledata = ''
portdata = ''                                                /* @PQA */

ipconn.0 = pipestem.0
allconn.0 = 0

do i = 1 to pipestem.0                            /* keyword loop    */
  /* EZL115I  IPCONN TCPIP* SP TIVED1                                */
  parse var pipestem.i . pipeappl pipeparm pipedata

  /* check if new IPCONN appl.                                       */
  Select                                          /* process IPCONNs */
   When conn_cnt = 0 & def_cnt = 0 then           /* 1st application?*/
    Do                                            /* process appl    */
        conn_cnt = conn_cnt + 1
        def_cnt = 0
        ipconn.conn_cnt.0 = def_cnt
        prevappl = pipeappl                       /* prev = curr     */
    End                                           /* process appl    */
   When pipeappl <> prevappl then                 /* new application?*/
    Do                                            /* process appl    */
      /* update counters to get array index values correct           */
        ipconn.conn_cnt.0 = def_cnt
        conn_cnt = conn_cnt + 1
        def_cnt = 0
        prevappl = pipeappl                       /* reset previous  */
    End                                           /* process appl    */
   Otherwise                                      /* more of same app*/
  End                                             /* process IPCONNs */

  /* now process the IPCONN keyword(s) & value(s)                    */
  select                                          /* process keywds  */
    when pipeparm = 'SP' then
      Do                                          /* process SP      */
        /* build string : IPCONN.1.1 = TCP* SP NMPIPL10              */
        def_cnt = def_cnt + 1
        all_cnt = all_cnt + 1
        ipconn.conn_cnt.def_cnt = pipeappl 'SP' pipedata
        allconn.all_cnt = pipeappl 'SP' pipedata
        /* Save stack in list to get community name later            */
        If POS(pipedata,StackList) = 0 then       /* new stack?      */
          StackList = StackList' 'pipedata
      End                                         /* process SP      */
    when pipeparm = 'LPORT' then                  /*                 */
      Do                                          /* process LPORT   */
        /* build string : IPCONN.1.2 = TCP* LPORT (23)               */
        def_cnt = def_cnt + 1
        all_cnt = all_cnt + 1
        ipconn.conn_cnt.def_cnt = pipeappl 'LPORT' pipedata
        allconn.all_cnt = pipeappl 'LPORT' pipedata
      End                                         /* process LPORT   */
    when pipeparm = 'MAXRECS' then                /*                 */
      Do                                          /* process MAXRECS */
        /* build string : IPCONN.1.n = TCP* MAXRECS 100              */
        def_cnt = def_cnt + 1
        all_cnt = all_cnt + 1
        ipconn.conn_cnt.def_cnt = pipeappl 'MAXRECS' pipedata
        allconn.all_cnt = pipeappl 'MAXRECS' pipedata
      End                                         /* process MAXRECS */
    when pipeparm = 'USEIPADDR' then              /*                 */
      Do                                          /* handle USEIPADDR*/
        /* build string : IPCONN.1.n = TCP* USEIPADDR NO             */
        def_cnt = def_cnt + 1
        all_cnt = all_cnt + 1
        ipconn.conn_cnt.def_cnt = pipeappl 'USEIPADDR' pipedata
        allconn.all_cnt = pipeappl 'USEIPADDR' pipedata
      End                                         /* handle USEIPADDR*/
    when pipeparm = 'THRESH' then
      Do                                          /* process THRESH  */
        /* build string : IPCONN.1.3 = TCP* THRESH (*,MIN,....)      */
        def_cnt = def_cnt + 1
        all_cnt = all_cnt + 1
        ipconn.conn_cnt.def_cnt = pipeappl 'THRESH' pipedata
        allconn.all_cnt = pipeappl 'THRESH' pipedata
      End                                         /* process THRESH  */
    otherwise
      Iterate
  End                                             /* process keywds  */
End                                               /* keyword loop    */


/* Now save last IPCONN entry                                        */

ipconn.conn_cnt.0 = def_cnt
allconn.0 = all_cnt
StackList = Strip(StackList)

If conn_cnt = 0 then                              /* no IPCONN defs  */
  Go_On = 'N'



Return




/*-------------------------------------------------------------------*/
/* Process_Policy:                                                   */
/* Loop through IPCONN policy definitions.                           */
/* - Use TCPCONN to determin the 4tuple data for the connections.    */
/* - Use SNMP MIBs for detailed connections data.                    */
/*-------------------------------------------------------------------*/
Process_Policy:

tuplcnt = 0
Do qqq = 1 to conn_cnt                     /* Loop thru all IPCONNs */
 /* Pull stack (SP) and local port (LPORT) info from array:
 TCPIP* SP TIVED1
 TCPIP* LPORT (23)
 TCPIP* THRESH (*,MIN,00:04:00,NOTIFY)
 TCPIP* THRESH (2147,MIN,00:00:30,NOTIFY)
 */
  'PIPE (STAGESEP' sep') STEM ipconn.'qqq'.' ,
    sep ' SEP ',
    sep ' STEM tempdata. ',                /* may need for appl nm  */
    sep ' LOC /SP / /LPORT / /MAXRECS / /USEIPADDR /',
    sep ' STEM poldata.'

  local_port = ''
  local_stack = ''
  local_addr  = ''
  local_appl  = ''

  maxrecs = ''
  useipaddr = ''
  port_string = ''

  temp = ''

  Do zzz = 1 to poldata.0                  /* Loop thru 1 IPCONN    */
    parse var poldata.zzz  pipeappl pipeparm pipedata
    local_appl = pipeappl
    Select                                 /* parse IPCONN data     */
      When pipeparm = 'SP' then
       Do                                  /* parse local stack     */
         local_stack = pipedata
         /* snmpdata = "CNAME="cname" SNMPNAME="snmp_name           */
         snmpdata = value(local_stack)
         parse var snmpdata . 'SNMPNAME='local_addr
       End                                 /* parse local stack     */
      When pipeparm = 'LPORT' then
       Do                                  /* parse local port      */
         parse var pipedata '('local_port')'
       End                                 /* parse local port      */
      When pipeparm = 'MAXRECS' then       /* TCPCONN MAXRECS       */
        maxrecs = pipedata
      When pipeparm = 'USEIPADDR' then     /* TCPCONN LADDR?        */
        useipaddr = pipedata
      Otherwise
    End                                    /* parse IPCONN data     */

  End                                      /* Loop thru 1 IPCONN    */

  /* validate SP                                                    */
  If local_stack = '' then                 /* no SP, use default    */
       Do                                  /* parse local stack     */
         local_stack = Default_Stack
       End                                 /* parse local stack     */
  /* snmpdata = "CNAME="cname" SNMPNAME="snmp_name           */
  snmpdata = value(local_stack)
  parse var snmpdata . 'SNMPNAME='local_addr

  /* validate MAXRECS                                               */
  If maxrecs = '' then                     /* no maxrecs? default   */
    MaxRecs = Default_MaxRecs

  /* validate USEIPADDR                                             */
  If useipaddr = '' then                   /* no useipaddr?         */
    useipaddr = Default_UseIpAddr          /* use default setting   */

  /* validate LPORT                                                 */
  If local_port = '' then                  /* no local port? default*/
   Do                                      /* no port, need appl    */
      parse var tempdata.1  pipeappl pipeparm pipedata
      Call Match_Appl
   End                                     /* no port, need appl    */

  CmdQueue.0 = 0
  /* Build and issue TCPCONN QUERY cmd, response -> TCPCONN. array  */
  CmdToIssue = TcpConn_Cmd||MaxRecs

  /* If USEIPADDR then add LADDR parm to TCPCONN command.           */
  /* Else, no LADDR is used and connection data will be retrieved   */
  /* for all local IP addrs on the stack (DVIPA, primary, etc.).    */
  If UseIpAddr = 'YES' then
    CmdToIssue = CmdToIssue' LADDR='local_addr

  Select                                   /* process local port(s) */
    When Pos(',',local_port) > 0 then      /* more than 1 port?     */
     Do                                    /* handle mult ports     */
       /* translate commas to blanks for better parsing             */
       port_String = Translate(local_port,' ',',')
       Do doh = 1 to Words(port_string)
         CmdQueue.Doh = CmdToIssue' LPORT='Word(port_String,doh)
       End
       CmdQueue.0 = Doh - 1
     End                                   /* handle mult ports     */
    When Pos(':',local_port) > 0 then      /* port range?           */
     Do                                    /* handle port range     */
       num_iterations = 0
       parse var local_port low_port':'high_port
       Curr_Port = low_port                /* start w/ lower port # */
       Do doh = low_port to high_port
         num_iterations = num_iterations + 1
         port_String = port_string' 'curr_port
         CmdQueue.num_iterations = CmdToIssue' LPORT='curr_port
         Curr_Port = Curr_port + 1         /* increment port #      */
       End
       CmdQueue.0 = num_iterations
     End                                   /* handle port range     */
    When Words(local_port) = 1 then        /* only 1 local port?    */
     Do                                    /* build 1 TCPCONN       */
      port_string = local_port
      CmdQueue.1 = CmdToIssue' LPORT='local_port
      CmdQueue.0 = 1
     End                                   /* build 1 TCPCONN       */
    Otherwise
  End                                      /* process local port(s) */

  port_string = Strip(port_string)

  tcpconn.0 = 0
  Do Doh = 1 to CmdQueue.0                 /* loop thru TCPCONN cmds*/
    'PIPE (STAGESEP' sep') NETV 'CmdQueue.Doh ,
      sep ' CORR ',
      sep ' SEP ',
      sep ' NLOC /BNH772/ ',               /* drop header msg       */
      sep ' NLOC /:/ ',                    /* drop any IPv6 addrs   */
      sep ' Stem TCPCONN. APPEND'
  End                                      /* loop thru TCPCONN cmds*/

  Do i = 1 to tcpconn.0                    /* Loop thru TCPCONN data*/
   Parse var tcpconn.i tc_appl tc_locaddr tc_lport ,
             tc_remaddr tc_rport .

   If Pos(tc_lport,port_string) > 0 then   /* port match?           */
    Do                                     /* build & save 4-tuple  */
      tuplcnt = tuplcnt + 1
      fourtuple.0 = tuplcnt
      fourtuple.tuplcnt = tc_locaddr'.'tc_lport||,
          '.'tc_remaddr'.'tc_rport' 'snmpdata
      temp = fourtuple.tuplcnt
    End                                    /* build & save 4-tuple  */

  End                                      /* Loop thru TCPCONN data*/


End                                        /* Loop thru all IPCONNs */

fourtuple.0 = tuplcnt

Do rr = 1 to tuplcnt                  /* now, loop thru the 4tuples  */
  temp = fourtuple.rr

  Call GetSNMPData temp               /* GET SNMP MIB data           */

 If Go_On = 'Y' then                  /* SNMP MIBs retrieved?        */
  Do
    Call Parse_MIB_Data               /* parse out connection data   */
    If Name.0 > 0 then                /* got data?                   */
      Call Apply_Thresholds           /* compare against policy      */
    Else                              /* no MIB data                 */
     Do
      Return_Code = 10                /* set no data return code     */
      'PIPE STEM SNMP_Msgs. ',
        sep ' CONS '
     End
  End

End                                   /* now, loop thru the 4tuples  */



Return

/*-------------------------------------------------------------------*/
/* Match_Appl:                                                       */
/* Retrieve the known ports, find those that match the IPCONN appl.  */
/*-------------------------------------------------------------------*/
Match_Appl:

Default_Lport = ''

Appls.0 = 0

Wild_card = 'N'
If pos('*',pipeappl) > 0 then          /* wildcard in name?          */
 Do
  testappl = substr(pipeappl,1,(length(pipeappl)-1))
  Wild_card = 'Y'
 End
else
  testappl = pipeappl

'PIPE STEM Port_Array. ',
  sep ' SEP ',
  sep ' LOC /'testappl'/ ',
  sep ' STEM Appls. '

Do x = 1 to Appls.0                    /* loop thru appls            */
  Parse var Appls.x portnum portappl
  If Wild_Card = 'Y' then              /* wildcard for appl name?    */
    /* Ck match for PREVAPPL minus the *, if no match then ignore    */
    If substr(portappl,1,length(testappl)) <> testappl then
      Iterate                          /* ignore this appl           */
  Default_Lport = Default_Lport','portnum
End                                    /* loop thru appls            */

Default_Lport = Strip(Default_Lport,'B',' ')
Default_Lport = Strip(Default_Lport,'B',',')
local_port = Default_Lport
/* Add LPORT to the allconn. array                                   */
All_Cnt = Allconn.0 + 1
Default_Lport = '('Default_Lport')'
AllConn.All_cnt = pipeappl' 'pipeparm' LPORT 'Default_Lport
AllConn.0 = all_cnt

Return

/*-------------------------------------------------------------------*/
/* GetSNMPData:                                                      */
/* Issue SNMP GETs to gather connection data details.                */
/*-------------------------------------------------------------------*/
GetSNMPData:
  fourtuple = temp

  Mib_cmd = Snmp_GET                  /* prepare SNMP GET command    */
  parse var fourtuple tuple ,
            'CNAME=' cname 'SNMPNAME=' snmp_name .
  tuple = Strip(tuple)

  if cname <> '' then                 /* community name defined?     */
    Mib_Cmd = Mib_Cmd '-c 'cname      /* concat into cmd string      */
  if snmp_name <> '' then             /* host ipaddr resolved?       */
    Mib_Cmd = Mib_Cmd '-h 'snmp_name  /* concat into cmd string      */

  g1 = ibmMvsTcpConnLastActivity||tuple
  g2 = ibmMvsTcpConnBytesIn||tuple
  g3 = ibmMvsTcpConnBytesOut||tuple
  g4 = ibmMvsTcpConnSndWnd||tuple
  g5 = ibmMvsTcpConnResourceName||tuple
  g6 = ibmMvsTcpConnResourceId||tuple

  Mib_Cmd = Mib_Cmd 'GET' g1 g2 g3 g4 g5 g6

  /* Execute SNMP GET ... -c ... -h .... g1 g2 g3 g4 g5 g6           */
  'PIPE NETV ' Mib_Cmd ,
    sep ' CORR ',
    sep ' STEM SNMP_Msgs. ',          /* hold all msgs in case of err*/
    sep ' DROP FIRST 1',
    sep ' SAFE FKXMIBS'

Return



/*-------------------------------------------------------------------*/
/* Parse_MIB_Data:                                                   */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Note: The ibmtcpipmvstcpconntable contains lots of detail (MIBs)  */
/* per connection that we do not need.                               */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Parse the response from the WALK request, looking for the 6 MIB   */
/* variables we need, ignoring the others.                           */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* After this point the 6 MIB vars are stored in their own arrays:   */
/*   Act. contains the LastActivity MIB data                         */
/*   In. contains the BytesIn MIB data                               */
/*   Out. contains the BytesOut MIB data                             */
/*   Name. contains the ResourceName MIB data (proc name!)           */
/*   Id. contains the ResourceId MIB data (connection id)            */
/*   Wnd. contains the Send Window MIB data (send window size)       */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Plus, all of the data for the 6 MIBs is stored into one array:    */
/*   Data. contains all required MIB data for this process.          */
/*-------------------------------------------------------------------*/
Parse_MIB_Data:

'PIPE (STAGESEP 'sep' END 'endit') SAFE FKXMIBS SEIZE',
      sep ' SEP',
      sep ' NLOC /'LoopBack'/ /'InAddrAny'/ /'BoundAddr'/' ,
      sep ' SPLITUP: FANOUT',           /* split into mult arrays ...*/
      sep ' LOC /'ibmMvsTcpConnLastActivity'/ ',
      sep ' STEM Act. ',                /* Act. : LastActivity MIBs  */
      endit ' SPLITUP:',
      sep ' LOC /'ibmMvsTcpConnBytesIn'/ ',
      sep ' STEM In. ',                 /* In. : BytesIn MIBs        */
      endit ' SPLITUP:',
      sep ' LOC /'ibmMvsTcpConnBytesOut'/ ',
      sep ' STEM Out. ',                /* Out. : BytesOut MIBs      */
      endit ' SPLITUP:',
      sep ' LOC /'ibmMvsTcpConnResourceName'/ ',
      sep ' STEM Name. ',               /* Name. : ResourceName MIBs */
      endit ' SPLITUP:',
      sep ' LOC /'ibmMvsTcpConnResourceId'/ ',
      sep ' STEM Id. ',                 /* Id. : ResourceId MIBs     */
      endit ' SPLITUP:',
      sep ' LOC /'ibmMvsTcpConnSndWnd'/ ',
      sep ' STEM Wnd. ',                /* Wnd. : Send Window MIBs   */
      endit ' SPLITUP:',
      sep ' LOC /'ibmMvsTcpConnLastActivity'/ ',
          ' /'ibmMvsTcpConnBytesIn'/ ',
          ' /'ibmMvsTcpConnBytesOut'/ ',
          ' /'ibmMvsTcpConnResourceName'/ ',
          ' /'ibmMvsTcpConnResourceId'/ ',
          ' /'ibmMvsTcpConnSndWnd'/ ',
      sep ' stem data.'                 /* Data. : store 4 all 6 MIBs*/


Return





/*-------------------------------------------------------------------*/
/* Apply_Thresholds:                                                 */
/* Compare connection data to IPCONN thresholds and take the         */
/* appropriate action(s).                                            */
/*-------------------------------------------------------------------*/
Apply_Thresholds:

/* Loop through the MIB data and call Compare_Data to perform the    */
/* threshold procesing.                                              */

do i = 1 to Name.0                      /* ResourceName MIB Loop     */
  /* init/clear vars to hold MIB data each time we start the loop    */
  LastActivity = ''
  BytesIn = ''
  BytesOut = ''
  ResourceName = ''
  ResourceId = ''
  LclPort = ''
  CliPort = ''
  Stack_Ref = ''
  Client_Ref = ''
  Stack_Tuple = ''
  Client_Tuple = ''
  Client_IP = ''
  SendWdw = ''

  /* parse out connection's proc name & determine if we care or not  */
  parse var Name.i MibVar '=' . MibVal
  ResourceName = Strip(MibVal)

  /* parse out stack ipaddr & port and client ipaddr & port          */
  /* ibmMvsTcpConnResourceName.9.67.50.52.23.146.84.147.16.1083 = 245*/
  parse var Name.i  '1.3.6.1.4.1.2.6.19.2.2.7.1.1.37.',
                    i1 '.' i2 '.' i3 '.' i4 '.' LclPort '.',
                    e '.' f '.' g '.' h '.' CliPort .
  LclPort = Strip(LclPort)
  CliPort = Strip(CliPort)
  Stack_Ref  = i1"."i2"."i3"."i4":"LclPort
  Client_Ip  = e"."f"."g"."h
  Client_Ref = Client_Ip":"CliPort
  Stack_Tuple = i1"."i2"."i3"."i4"."LclPort
  Client_Tuple = Client_Ip"."CliPort

      parse var Act.i MibVar '=' MibVal
      LastActivity = Strip(MibVal)
      /* LastActivity is a timeticks format in hundredths of a sec   */
      /* So we need to perform an integer divide by 100              */
      LastActivity = LastActivity % 100         /* num seconds now!  */

      parse var In.i MibVar '=' . MibVal
      BytesIn = Strip(MibVal)
      BytesIn = Strip(BytesIn,'L','0')          /* strip leading 0   */

      parse var Out.i MibVar '=' . MibVal
      BytesOut = Strip(MibVal)
      BytesOut = Strip(BytesOut,'L','0')        /* strip leading 0   */

      parse var Id.i MibVar '=' . MibVal
      ResourceId = Strip(MibVal)
      ConnId = D2X(ResourceId,10)               /* save hex equiv too*/

      parse var Wnd.i MibVar '=' . MibVal
      SendWdw = Strip(MibVal)

      /* compare MIB data against policy specifications now.         */
      Call Compare_Data
End                                     /* ResourceName MIB Loop     */


Return




/*-------------------------------------------------------------------*/
/* Compare_Data:                                                     */
/* Compare MIB var data such as LastActivity against IPCONN          */
/* threshold(s) such as IDLETIME.                                    */
/*-------------------------------------------------------------------*/
Compare_Data:

/* Two Passes needed here:                                           */
/* 1)Index into ipconn.i to find the correct IPCONN definition that  */
/* matches the local port from the MIB data.                         */
/*                                                                   */
/* 2)Parse IPCONN policy name (1st word) and pull all of the IPCONN  */
/* policy from ipconn.i array.                                       */
/*                                                                   */
/* THRESH line(s):  (zero or more)                                   */
/*   1st word = IPCONN policy name                                   */
/*   2nd word = THRESH                                               */
/*   remainder= (byte_cnt,thresh_type,interval,action1,action2)      */
/*                                                                   */
/* LPORT line:                                                       */
/*   1st word = IPCONN policy name                                   */
/*   2nd word = LPORT                                                */
/*   remainder= (port_num)                                           */
/*   => OR <=   (port_num,...,port_num)                              */
/*   => OR <=   (port_num:port_num)                                  */

Notify = ''
Notify_SP = ''
Policy_Name = ''
IpconnData.0 = 0

/* 1st pass - find policy name that matches local port              */
/* TCPIP* LPORT (23)                                                */
'PIPE STEM allconn.' ,
  sep 'LOC /LPORT /' ,                 /* gotta LOC all LPORT 1st   */
  sep 'LOC /('lclport')/' ,            /* single port num           */
      '    /('lclport',/' ,            /* start of mult ports       */
      '    /,'lclport',/' ,            /* middle of mult ports      */
      '    /,'lclport')/' ,            /* end of mult ports         */
  sep 'VAR Ipconn_Data'

If Symbol('Ipconn_Data') = 'VAR' then  /* found port number?        */
  parse var Ipconn_Data Policy_Name .
Else                                   /* pass "1A" - ck range      */
  Do                                   /* port could be in a range  */
    'PIPE STEM allconn.' ,
      sep 'LOC /LPORT /' ,             /* retrieve all LPORT ...    */
      sep 'LOC /:/' ,                  /* ... with a range spec ... */
      sep 'STEM port_ranges.'          /* ... into another array    */
    Do rcnt = 1 to port_ranges.0       /* loop thru ranges & compare*/
      Parse var Port_Ranges.rcnt . '('lower_range':'upper_range')'
      If lclport >= lower_range &,
        lclport <= upper_range then    /* port is in the range      */
           /* found correct port range, parse out policy name       */
           parse var port_ranges.rcnt Policy_Name .
    End                                /* loop thru ranges & compare*/
  End                                  /* port could be in a range  */


/* 2nd pass - retrieve full IPCONN policy for local port            */
'PIPE (STAGESEP 'sep' END 'endit') STEM allconn.' ,
  sep 'LOC /'Policy_Name '/' ,         /* loc only IPCONN 4 lclport */
  sep 'NLOC /LPORT / /MAXRECS /' ,     /* do not need these ...     */
  sep ' SPLITUP: FANOUT',           /* split into mult arrays ...*/
  sep 'LOC /'Policy_Name' SP /' ,   /* loc SP keyword ...        */
  sep 'VAR Notify_SP ' ,               /* store in variable         */
  endit ' SPLITUP:',
  sep 'LOC /'Policy_Name' THRESH/' ,/* loc THRESH keyword(s) ... */
  sep 'STEM IpconnData.'               /* store in array            */


/* parse & apply IPCONN policy specification for name match          */
/* The format of Ipconndata.xx is similar to:
Notify_SP = TCPIP* SP TIVED1
IpconnData.1 = TCPIP* THRESH (*,MIN,00:04:00,NOTIFY)
IpconnData.2 = TCPIP* THRESH (2147,MIN,00:00:30,NOTIFY)
*/

If Symbol('Notify_SP') = 'LIT' then     /* null?                      */
  Notify_SP = Default_Stack
Else
  Parse var Notify_SP . keyword Notify_SP
Notify_SP = Strip(Notify_SP)

Do xx = 1 to IpconnData.0              /* loop thru IPCONN policy    */
  Parse var IpconnData.xx . keyword keyval
  thresh = keyval
  parse var thresh '('byte_cnt','thresh_type','thresh_int,
            ','actions')'
  If actions = '' then                             /* no actions?    */
    thresh_int = Strip(thresh_int,'T',')')         /* remove )       */
  /* convert hh:mm:ss to seconds for comparison to LastActivity      */
  parse var thresh_int th':'tm':'ts
  thresh_int = ts + (60*tm) + (60*60*th)           /* need seconds   */

  Select                                           /* compare ...    */
    When byte_cnt = '*' &,                         /* IDLETIME thresh*/
         LastActivity > thresh_int then
      Do                                           /* process IDLE   */
        /* Connection is idle - no activity has occurred recently.   */
        Notify = 'IDLE'
      End                                          /* process IDLE   */
    When thresh_type = 'MIN' &,                    /* MINBYTES thresh*/
         SendWdw = 0 Then
      Do                                           /* process MIN    */
        /* SendWdw = 0 says connection is hung ... don't bother      */
        /* checking the LastActivity and BytesOut data.              */
        Notify = 'MIN'
      End                                          /* process MIN    */
    When thresh_type = 'MIN' &,                    /* MINBYTES thresh*/
         thresh_int < LastActivity &,
         byte_cnt > BytesOut then
      Do                                           /* process MIN    */
        /* Connection could be hung - setup to take action(s).       */
        Notify = 'MIN'
      End                                          /* process MIN    */
    When thresh_type = 'MAX' &,                    /* MAXBYTES thresh*/
         thresh_int < LastActivity &,
         byte_cnt < BytesOut then
      Do                                           /* process MAX    */
        Notify = 'MAX'
      End                                          /* process MAX    */
     Otherwise
  End                                              /* compare ...    */

  If Notify <> '' then
   Do
    If actions <> '' &,                            /* 1 or more acts?*/
       pos('NONE',Actions) = 0 then                /* but no NONE?   */
             Call Take_Action
    /* Action taken for a THRESH statement. No further checks done.  */
    xx = IpconnData.0                              /* end loop now   */
   End
End                                    /* loop thru IPCONN policy    */



Return



/*-------------------------------------------------------------------*/
/* Take_Action:                                                      */
/* Parse THRESH action(s) & act appropriately.                       */
/*-------------------------------------------------------------------*/
Take_Action:

Action = ''
LogMsg = 'N'
IssueMsg = 'N'
actions = translate(actions,' ',',')

Do s = 1 to words(actions)            /* Loop thru action(s)         */
  If word(Actions,s) = 'NOTIFY' |,
     word(Actions,s) = 'DROP' then    /* process NOTIFY | DROP  @P2A */
     If pos(word(Actions,s),Action) = 0 then /* new action?     @P5A */
      Action = Action||word(Actions,s)/* concat actions         @P2C */
     /* Else, duplicate action coded - ignore it                @P5A */
End                                   /* Loop thru action(s)    @P2M */


  /* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
  /* At this point Action can be:                                    */
  /*  NONE (should not occur, because if NONE we can't get here)     */
  /*  NOTIFY                                                         */
  /*  DROP                                                           */
  /*  NOTIFYDROP                                                     */
  /*  DROPNOTIFY                                                     */
  /* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
  /* When Action is NONE    : take no action at all                  */
  /* When Action is DROP    :                                        */
  /*      If "idle" then break the connection                        */
  /*      If "max" then break the connection                    @P4C */
  /*      If "hung" then break the connection                        */
  /*      Else, ignore the connection data until SendWdw is 0.       */
  /* When Action is NOTIFY  : issue message (via Process_Msg Rtn)    */
  /* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */

  Select                              /* determine which action      */
    When Pos('NONE',Action) > 0 then  /* no action - get outta here  */
                                      /* ... precautionary!     @P2C */
      Return                          /* to exit, end loop now  @P2C */

    When Pos('DROP',Action) > 0 &,    /*                        @P2C */
        (Byte_Cnt = '*' |,            /* IDLE session ?              */
         Notify = 'MAX') then         /* MAX thresh exceeded?   @P4C */
      Do                              /* process DROP                */
        Call Break_Conn               /* DROP all idle connections   */
        Call Process_Msg              /* issue or log message        */
                                      /* setup to Log message   @P2D */
      End                             /* process DROP                */

    When Pos('DROP',Action) > 0 &,    /*                        @P2C */
         SendWdw = 0 then             /* send window is closed       */
      Do                              /* process DROP                */
        /* Only DROP those connections whose SendWdw is 0            */
        Call Break_Conn
        Call Process_Msg              /* issue or log message        */
                                      /* setup to Log message   @P2D */
      End                             /* process DROP                */

    When Pos('DROP',Action) > 0 &,
         SendWdw > 0 then             /* send window is still open   */
      Do                              /* Can't DROP yet              */
        /* For those connections whose SendWdw is not 0 ...          */
        /* don't DROP, but might need to NOTIFY                6@P2C */
        If Pos('NOTIFY',Action) > 0 then/* need to notify?           */
         Do                           /*                        @P2A */
          Action = 'NOTIFY'
          Call Process_Msg            /* issue or log message        */
         End                          /*                        @P2A */
        Else                          /* else, take no action        */
          Action = 'NONE'
                                      /* setup to Log message   @P2D */
     End                              /* Can't DROP yet              */

    Otherwise
        Call Process_Msg              /* issue or log message        */

  End                                 /* determine which action      */
    /* Notify Action moved to Process_Msg logic with   6@P2D         */

/*                                                             4@P2D */


Return


/*-------------------------------------------------------------------*/
/* Process_Msg:                                                      */
/* Issue or log message ...                                          */
/* (Following commentary added for @P2 to document changes)          */
/* Looks at the Threshold Type (Notify var) and Action type          */
/* (Action var) to determine which message to issue and if it        */
/* should be routed to a NTFYOP (NtfyParm var).                      */
/*-------------------------------------------------------------------*/
Process_Msg:

Conn_Ref = ResourceId'/'ConnId        /* set connection info parm    */
Conn_Ref = "'"Conn_Ref"'"
Stack_Ref = "'"Stack_Ref"'"
Client_Ref = "'"Client_Ref"'"
/*NtfyParm = IssueMsg                  * set SLN NOTIFY=Y|N     @P2D */

Resname = ResourceId                  /* set SLN resname parm        */
Resname = Conn_Ref                    /* set SLN resname parm        */
Restype = 'IPCONN'                    /* set SLN restype parm        */

Select                                       /* Set message text var */
  When Notify = 'IDLE' then                  /* Idle Thresh?    @P2A */
    Select                                   /* IDLE ACTIONS    @P2A */
      When Action = 'NOTIFY' then            /*                 @P2C */
       Do                                    /* issue FKX101    @P2A */
        NtfyParm = 'Y'                       /*set SLN NOTIFY=Y @P2A */
        Call Error_Msg101
       End                                   /* issue FKX101    @P2A */
      When Action = 'DROP' then
       Do                                    /* log FKX102      @P2A */
        NtfyParm = 'N'                       /*set SLN NOTIFY=N @P2A */
        Call Error_Msg102
       End                                   /* log FKX102      @P2A */
      When Action = 'DROPNOTIFY' |,
           Action = 'NOTIFYDROP' then        /*                 @P2A */
       Do                                    /* issue FKX102    @P2A */
        NtfyParm = 'Y'                       /*set SLN NOTIFY=Y @P2A */
        Call Error_Msg102                    /*                 @P2A */
       End                                   /* issue FKX102    @P2A */
      Otherwise
    End                                      /* IDLE ACTIONS    @P2A */
  When Notify = 'MIN' then                   /* Min Thresh?     @P2A */
    Select                                   /* MIN  ACTIONS    @P2A */
      When Action = 'NOTIFY' then            /*                 @P2C */
       Do                                    /* issue FKX104    @P2A */
        NtfyParm = 'Y'                       /*set SLN NOTIFY=Y @P2A */
        Call Error_Msg104
       End                                   /* issue FKX104    @P2A */
      When Action = 'DROP' then
       Do                                    /* log FKX105      @P2A */
        NtfyParm = 'N'                       /*set SLN NOTIFY=N @P2A */
        Call Error_Msg105
       End                                   /* log FKX105      @P2A */
      When Action = 'DROPNOTIFY' |,
           Action = 'NOTIFYDROP' then        /*                 @P2A */
       Do                                    /* issue FKX105    @P2A */
        NtfyParm = 'Y'                       /*set SLN NOTIFY=Y @P2A */
        Call Error_Msg105                    /*                 @P2A */
       End                                   /* issue FKX105    @P2A */
      Otherwise
    End                                      /* MIN  ACTIONS    @P2A */
  When Notify = 'MAX' then                   /* Max Thresh?     @P2A */
    Select                                   /* MAX  ACTIONS    @P2A */
      When Action = 'NOTIFY' then            /*                 @P2C */
       Do                                    /* issue FKX107    @P2A */
        NtfyParm = 'Y'                       /*set SLN NOTIFY=Y @P2A */
        Call Error_Msg107
       End                                   /* issue FKX107    @P2A */
      When Action = 'DROP' then
       Do                                    /* log FKX108      @P2A */
        NtfyParm = 'N'                       /*set SLN NOTIFY=N @P2A */
        Call Error_Msg108
       End                                   /* log FKX108      @P2A */
      When Action = 'DROPNOTIFY' |,
           Action = 'NOTIFYDROP' then        /*                 @P2A */
       Do                                    /* issue FKX108    @P2A */
        NtfyParm = 'Y'                       /*set SLN NOTIFY=Y @P2A */
        Call Error_Msg108                    /*                 @P2A */
       End                                   /* issue FKX108    @P2A */
      Otherwise
    End                                      /* MAX  ACTIONS    @P2A */
  Otherwise                                  /* No messages!         */
End                                          /* Set message text var */

Conn_Ref = ''                         /* reset connection info parm  */
NtfyParm = ''                         /* reset SLN NOTIFY=Y|N   @P2A */


Return


/*-------------------------------------------------------------------*/
/* Break_Conn:                                                       */
/* This is done via an SNMP SET of the corresponding tcpConnState    */
/* MIB var to a value of 12.                                         */
/*-------------------------------------------------------------------*/
Break_Conn:


Mib_Cmd = snmp_SET                    /* prepare SNMP SET command    */

if cname <> '' then                   /* community name defined?     */
  Mib_Cmd = Mib_Cmd '-c 'cname        /* concat into cmd string      */
if snmp_name <> '' then               /* host ipaddr resolved?       */
  Mib_Cmd = Mib_Cmd '-h 'snmp_name    /* concat into cmd string      */

Set_Req = tcpConnState'.'stack_tuple'.'client_tuple' 'mibtype' 12'
Mib_Cmd = Mib_Cmd Set_Req

/* Execute SNMP SET ... -c ... -h .... tcpConnState ...              */
'PIPE NETV ' Mib_Cmd ,
  sep ' CORR ',
  sep ' STEM SNMP_Msgs. ',            /* hold all msgs in case of err*/
  sep ' CONS '                        /* display/log 'em too         */

Snmp_SET = 'snmp SET -v2c -Obestn'     /* reset SNMP SET for next    */

Return


/*-------------------------------------------------------------------*/
/* Reschedule_Timer:                                                 */
/* Reschedule this Clist using the AFTER command.                    */
/*-------------------------------------------------------------------*/
Reschedule_Timer:

'EZLEADLY 'interval',ID=FKXECMON,FKXECMON ' Default_stack


Return


/*-------------------------------------------------------------------*/
/* Get_Host_By_Name:                                                 */
/* We were passed a hostname only.  We need to translate the         */
/* hostname into an IP addr.                                         */
/*-------------------------------------------------------------------*/
Get_Host_By_Name:

snmp_name = FKXEGHBN(sp_hname)

Return




/*-------------------------------------------------------------------*/
/* Error routines & Messages                                         */
/*-------------------------------------------------------------------*/

Error_Msg101:
/*-------------------------------------------------------------------*/
/* FKX101I IDLE TIME THRESHOLD EXCEEDED FOR CONNECTION &&6 BETWEEN   */
/*         &&7 AND &&8. ACTION=NOTIFY SP=&&5 POLICY=&&9.             */
/*-------------------------------------------------------------------*/

If AON_Enabled = 'N' then
 Do
  Msg_Text = 'FKX101I IDLE TIME THRESHOLD EXCEEDED FOR',
           'CONNECTION' conn_ref,
           'BETWEEN' stack_ref 'AND' client_ref,
           'ACTION='action 'SP='Notify_sp,
           'POLICY='policy_name'.'
  Say Msg_Text
 End
Else
 "EZLEASLN NOTIFY="NtfyParm",STATUS=N,"Tower",THRESH,"Ident",FKX101,"||,
     Date","Time","ResName","ResType","||,
     sp","||,
     conn_ref","||,
     stack_ref","client_ref","||,
     policy_name


Return

Error_Msg102:
/*-------------------------------------------------------------------*/
/* FKX102I IDLE TIME THRESHOLD EXCEEDED FOR CONNECTION &&6 BETWEEN   */
/*         &&7 AND &&8. ACTION=DROP SP=&&5 POLICY=&&9.               */
/*-------------------------------------------------------------------*/

If AON_Enabled = 'N' then
 Do
  Msg_Text = 'FKX102I IDLE TIME THRESHOLD EXCEEDED FOR',
           'CONNECTION' conn_ref,
           'BETWEEN' stack_ref 'AND' client_ref,
           'ACTION='action 'SP='Notify_sp,
           'POLICY='policy_name'.'
  Say Msg_Text
 End
Else
"EZLEASLN NOTIFY="NtfyParm",STATUS=N,"Tower",THRESH,"Ident",FKX102,"||,
     Date","Time","ResName","ResType","||,
     sp","||,
     conn_ref","||,
     stack_ref","client_ref","||,
     policy_name

Return

Error_Msg104:
/*-------------------------------------------------------------------*/
/* FKX104I MINIMUM BYTES THRESHOLD EXCEEDED FOR CONNECTION &&6       */
/*         BETWEEN &&7 AND &&8. ACTION=NOTIFY SP=&&5 POLICY=&&9.     */
/*-------------------------------------------------------------------*/

If AON_Enabled = 'N' then
 Do
  Msg_Text = 'FKX104I MINIMUM BYTES THRESHOLD EXCEEDED FOR',
           'CONNECTION' conn_ref,
           'BETWEEN' stack_ref 'AND' client_ref,
           'ACTION='action 'SP='Notify_sp,
           'POLICY='policy_name'.'
  Say Msg_Text
 End
Else
"EZLEASLN NOTIFY="NtfyParm",STATUS=N,"Tower",THRESH,"Ident",FKX104,"||,
     Date","Time","ResName","ResType","||,
     sp","||,
     conn_ref","||,
     stack_ref","client_ref","||,
     policy_name

Return

Error_Msg105:
/*-------------------------------------------------------------------*/
/* FKX105I MINIMUM BYTES THRESHOLD EXCEEDED FOR CONNECTION &&6       */
/*         BETWEEN &&7 AND &&8. ACTION=DROP SP=&&5 POLICY=&&9.       */
/*-------------------------------------------------------------------*/

If AON_Enabled = 'N' then
 Do
  Msg_Text = 'FKX105I MINIMUM BYTES THRESHOLD EXCEEDED FOR',
           'CONNECTION' conn_ref,
           'BETWEEN' stack_ref 'AND' client_ref,
           'ACTION='action 'SP='Notify_sp,
           'POLICY='policy_name'.'
  Say Msg_Text
 End
Else
"EZLEASLN NOTIFY="NtfyParm",STATUS=N,"Tower",THRESH,"Ident",FKX105,"||,
     Date","Time","ResName","ResType","||,
     sp","||,
     conn_ref","||,
     stack_ref","client_ref","||,
     policy_name

Return

Error_Msg107:
/*-------------------------------------------------------------------*/
/* FKX107I MAXIMUM BYTES THRESHOLD EXCEEDED FOR CONNECTION &&6       */
/*         BETWEEN &&7 AND &&8. ACTION=NOTIFY SP=&&5 POLICY=&&9.     */
/*-------------------------------------------------------------------*/

If AON_Enabled = 'N' then
 Do
  Msg_Text = 'FKX107I MAXIMUM BYTES THRESHOLD EXCEEDED FOR',
           'CONNECTION' conn_ref,
           'BETWEEN' stack_ref 'AND' client_ref,
           'ACTION='action 'SP='Notify_sp,
           'POLICY='policy_name'.'
  Say Msg_Text
 End
Else
"EZLEASLN NOTIFY="NtfyParm",STATUS=N,"Tower",THRESH,"Ident",FKX107,"||,
     Date","Time","ResName","ResType","||,
     sp","||,
     conn_ref","||,
     stack_ref","client_ref","||,
     policy_name

Return

Error_Msg108:
/*-------------------------------------------------------------------*/
/* FKX108I MAXIMUM BYTES THRESHOLD EXCEEDED FOR CONNECTION &&6       */
/*         BETWEEN &&7 AND &&8. ACTION=DROP SP=&&5 POLICY=&&9.       */
/*-------------------------------------------------------------------*/

If AON_Enabled = 'N' then
 Do
  Msg_Text = 'FKX108I MAXIMUM BYTES THRESHOLD EXCEEDED FOR',
           'CONNECTION' conn_ref,
           'BETWEEN' stack_ref 'AND' client_ref,
           'ACTION='action 'SP='Notify_sp,
           'POLICY='policy_name'.'
  Say Msg_Text
 End
Else
"EZLEASLN NOTIFY="NtfyParm",STATUS=N,"Tower",THRESH,"Ident",FKX108,"||,
     Date","Time","ResName","ResType","||,
     sp","||,
     conn_ref","||,
     stack_ref","client_ref","||,
     policy_name

Return



Error_Msg651:
/*-------------------------------------------------------------------*/
/* DSI651I KEYWORD MISSING - ONE OF FOLLOWING REQUIRED: keywords     */
/*-------------------------------------------------------------------*/
 EzlMsgs.0 = 1
 EzlMsgs.1 = ezlemsg('DSI651','N','IPADDR','HOSTNAME')
 ExitMsg = EzlMsgs.1
 Return_code = 3
 call LEAVE_NOW
Return


/*-------------------------------------------------------------------*/
/* EZL203I   EXPECTED PARAMETERS MISSING FOR REQUEST  ...            */
/*-------------------------------------------------------------------*/
ERROR_MSG203:

 EXITMSG = EZLEMSG(203,'N',Ident,Badparm)
 Return_Code = 4
 call Leave_now


/*-------------------------------------------------------------------*/
/* EZL204I   EXPECTED PARAMETERS INVALID FOR REQUEST ...             */
/*-------------------------------------------------------------------*/
ERROR_MSG204:

 EXITMSG = EZLEMSG(204,'N',Ident,Badparm,BadParmval)
 Return_Code = 4
 call Leave_now


/*-------------------------------------------------------------------*/
/* EZL205I   cmd  COMMAND FAILED ... - WAIT TIME EXPIRED             */
/*-------------------------------------------------------------------*/
ERROR_MSG205:

 EXITMSG = EZLEMSG(205,'N',Ident,Badcmd,Badcmd,Waittime)
 Return_Code = 5
 call Leave_now

/*-------------------------------------------------------------------*/
/* EZL206I   cmd COMMAND FAILED ... RECEIVED "rc"                    */
/* Errmsg                                                            */
/*-------------------------------------------------------------------*/
ERROR_MSG206:

 EXITMSG = EZLEMSG(206,'N',Ident,Badcmd,Badcmdrc,Errmsg)
 Return_Code = 5
 call Leave_now

/*-------------------------------------------------------------------*/
/* CLIST clist FAILED : LINE Sigl HAD A VARIABLE WITH NO VALUE       */
/*-------------------------------------------------------------------*/
NOVALUE:

 EXITMSG = EZLEMSG(271,'Y',ident,sigl,Condition('D'))
 Return_Code = 7                       /* return to caller           */
 call Leave_now

/*-------------------------------------------------------------------*/
/* Ident COMMAND FAILED FOR REXX : RECEIVED RETCODE=SYNTAX ERROR     */
/* BadRC on line Sigl                                                */
/*-------------------------------------------------------------------*/
SYNTAX:

 BadRC = RC
 EXITMSG = EZLEMSG(275,'Y',ident,sigl,BadRC)
 Return_code = 8

 call Leave_now

/*-------------------------------------------------------------------*/
/* GET OUT!!!!!                                                      */
/*-------------------------------------------------------------------*/
FAILURE:
   EZL_PFEXIT = 'PF2'
   'GLOBALV PUTT EZL_PFEXIT'
  'EZLTRACE EXIT 'ident '-1'
  Exit -1

HALT:
  EZL_PFEXIT = 'PF2'
  'GLOBALV PUTT EZL_PFEXIT'
  'EZLTRACE EXIT 'ident '-5'
 Exit -5


/*-------------------------------------------------------------------*/
/* Exit Trace Call                                                   */
/*-------------------------------------------------------------------*/
Leave_now:
 /* If Domain E/E trace is 'ON' or Oper E/E trace 'ON' ...           */
 If Substr(EZLTRACED,1,2) = 'ON' |,          /* If Domain on ..      */
    (Substr(EZLTRACED,1,3) = 'OFF' & OperTrace = 'ON') Then /*       */
   'EZLTRACE EXIT 'Ident Return_code         /* Call Exit Trace      */

   If EXITMSG <> '' then
     Say ExitMsg                                            /*  @P3A */
                                                            /* 2@P3D */

 Exit Return_code