/*********************************************************************/
/*                                                                   */
/* EXEC NAME       : GETPORTS                                        */
/*                                                                   */
/* DESCRIPTIVE NAME: REXX EXEC to retrieve TCP/IP port data.         */
/*                                                                   */
/* FUNCTION        : Retrieve TCP/IP port data (port number and      */
/*                   application).                                   */
/*                   Update in storage file with that data.          */
/*                   Place data into a named safe (filename).        */
/*                                                                   */
/* NOTES           : Invoked as                                      */
/*                   GETPORTS interval stack_name                    */
/*                     interval = hh:mm                              */
/*                     stack_name = TCP390 definition                */
/*                                                                   */
/*                 : Customer modifiable variables are located in    */
/*                   CUSTOMER_VARS                                   */
/*                                                                   */
/* Return Codes    : RC =  0 All is well!                            */
/*                   RC =  3 IPADDR &/or HOSTNAME missing            */
/*                   RC =  4 Missing or invalid parms                */
/*                   RC =  9 Security authorization failure          */
/*                           (See FKXESCMD for further details)      */
/*                   RC = 10 No MIB Data                             */
/*                   All others = See FKXESCMD for further details   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/* DEPENDENCIES    : NetView PIPEs                                   */
/*                   SNMP V1 or V2                                   */
/*                                                                   */
/* MESSAGES        : TBD.                                            */
/*                                                                   */
/* OPERATION       : Same as function above                          */
/*                                                                   */
/* GLOBALS         :   EZLTRACED - Domain Trace Global               */
/*                 :   EZLTRACEO.opid - Operator Trace Global        */
/*                 :   CNMSTYLE.CONNMONPORTS                         */
/*                 :   CNMSTYLE.CONNMONFILE                          */
/*                                                                   */
/*                                                                   */
/* CHANGE ACTIVITY :                                                 */
/* APAR#    DATE       ID DESCRIPTION                                */
/* -------- --------   -- -----------------------------------------  */
/* $D1=Dxxxxxxx,TIVNV52,10/11/06,PJQ: CONNMON prototype              */
/*                                                                   */
/*                                                                   */
/*-------------------------------------------------------------------*/
 Trace Off
 parse source . Invoc Ident .
 parse arg argstring                                 /* set argstring*/
 Date = date('U')                                    /* mm/dd/yy fmt */
 Time = substr(time(),1,5)                           /* hh:mm fmt    */
 /* Bypass Security since this is "internal" processing              */
 Call AUTBYPAS('ON')

/*-------------------------------------------------------------------*/
/* Entry Trace Call                                                  */
/*-------------------------------------------------------------------*/
 'GLOBALV GETC EZLTRACED EZLTRACEO.'Opid()   /* Trace Variable       */

If Substr(EZLTRACED,1,4) <> 'NONE' Then  /* If not NONE ...          */
  Do
   'EZLTRACE ENTRY 'ident argstring                  /* Entry trace  */
   interpret 'trace' EZLTRACER
  End
OperTrace = Substr(Value('EZLTRACEO.'Opid()),1,2)

signal on NOVALUE                                    /* RAS signals  */
signal on SYNTAX
signal on HALT
signal on FAILURE

ADDRESS NETVASIS

/*-------------------------------------------------------------------*/
/* Main Program                                                      */
/* Make sure you set: Tower, Resource_name, Resource_type, Badcmd    */
/*                    Errmsg BadParm BadParmVal Return_code          */
/*-------------------------------------------------------------------*/
Call Init
Call Customer_Vars

Call Parse_Validate_Input
Call Get_Cname Default_Stack

Call Get_IPPorts

Call Log_Ports
/*
Say 'INSTORE file contains:'
'PIPE < DSIPARM.'filename ,
  ' | CONS'
*/
Call Leave_now                                       /*tidy exit     */




Customer_Vars:
/*-------------------------------------------------------------------*/
/* Customer_Vars:                        = = = >   L 0 0 K   < = = = */
/* Customers can modify these variables.                             */
/*-------------------------------------------------------------------*/

/* MaxPorts value is used on SNMP GETBULK command: -Cr<MaxPorts>.    */
/* By default this setting will retrieve data for "up to" 100 ports. */
/* Adjust higher if you are running more than 100 ports.             */
/* Adjust lower if you are running less than 100 ports. Otherwise,   */
/* extraneous data will be retrieved with the GETBULK and ignored in */
/* the GETBULK PIPE. Extraneous data means additional cycles.        */
MaxPorts = 100


/* Define a default SNMP community name in case it it not defined on */
/* a stack policy definiton (TCP390). The preferred method is to use */
/* the COMMUNITYNAME keyword on your TCP390 definition.              */
Default_Cname    = 'publicv2c'


Return


Init:
/*-------------------------------------------------------------------*/
/* Init:                                                             */
/* Initialize all local clist vars                                   */
/*-------------------------------------------------------------------*/

Go_On = 'Y'
Errormsg = ''
BadParm  = ''
ExitMsg = ''
Return_Code = 0

sp = ''
spdata = ''
Default_Stack = ''
Default_filename = 'CONNMON'
Default_CollType = 'STATIC'
interval = ''

sep = '|'                             /* PIPE stage separator char   */
endit = '?'                           /* PIPE END stage character    */

/*- - - - - - - - - - - MIB  Definitions  - - - - - - - - - - - - - -*/
ibmMvsTcpListenerLocalPort = '.1.3.6.1.4.1.2.6.19.2.2.7.31.1.4'
ibmMvsTcpListenerResourceName = '.1.3.6.1.4.1.2.6.19.2.2.7.31.1.12'
/*- - - - - - - - - - - MIB  Definitions  - - - - - - - - - - - - - -*/


Return



Parse_Validate_Input:
/*-------------------------------------------------------------------*/
/* Parse_Validate_Input:                                             */
/* Parse input parameters and validate them.                         */
/*-------------------------------------------------------------------*/

Parse Var Argstring Default_Stack .

'GLOBALV GETC CNMSTYLE.CONNMONPORTS ',
            ' CNMSTYLE.CONNMONFILE'

/* Save filename.  If none defined use default                        */
filename = value(CNMSTYLE.CONNMONFILE)
If filename = '' then
  filename = Default_filename

/* Check type of port collection defined in CNMSTYLE.CONNMONPORTS:    */
/*  - If STATIC then port data is collected once at initialization    */
/*  - If DYNAMIC then port data is collected very time connection
      monitoring is scheduled.                                        */
/*  - If semi-DYNAMIC then port data is scheduled independent of
      connection monitoring.                                          */
/*    Semi-DYNAMIC means global contains interval (DYNAMIC,hh:mm:ss)  */
Parse var CNMSTYLE.CONNMONPORTS '('CollType','CollIntvl')'

/* CollType = value(CNMSTYLE.CONNMONPORTS) */
If CollType = '' then                  /* No collection type def'd?   */
  CollType = Default_CollType          /* Use default (STATIC)        */
Else                                   /* Collection type defined     */
  Do                                   /* validate collection type    */
    Select
     When CollType = 'STATIC' then     /* STATIC ?                    */
       Nop                             /* Take no further action      */
     When CollType = 'DYNAMIC' then    /* DYNAMIC ?                   */
      Do                               /* Ck for interval ...         */
       If CollIntvl <> '' then         /* Collection interval def'd?  */
        Do                             /* Validate interval           */
          Interval = CollIntvl
          Call Validate_Interval
        End                            /* Validate interval           */
      End                              /* Ck for interval ...         */
     When CollType = 'DYNAMIC)' then   /* DYNAMIC ?                   */
      CollType = 'DYNAMIC'
     Otherwise                         /* ERROR - bad type            */
      Do                               /* Invalid collection type     */
       Say 'CONNMON - Invalid collection type definition:' CollType
       CollType = Default_CollType     /* default to STATIC           */
      End                              /* Invalid collection type     */
  End                                  /* validate collection type    */

/* test code */
If Default_Stack = '' then Default_Stack = 'TIVED1'

Return


/*-------------------------------------------------------------------*/
/* Validate_Interval:                                                */
/* Validate the interval from the ACTMON IPCONN policy def.          */
/*-------------------------------------------------------------------*/
Validate_Interval:

If Interval = '' then                 /* If still no interval ...    */
  Do                                  /* issue EZL203I message       */
    Badparm = 'INTVL'                 /*                        @P3C */
    Call ERROR_MSG203
  End                                 /* issue EZL203I message       */

parse var interval hh ':' mm ':' ss
hh = right(hh,2,'0')
mm = right(mm,2,'0')
ss = right(ss,2,'0')


Select
  When hh > 24 | hh < 00 | datatype(hh) <> 'NUM' then
    Call ERROR_MSG203
  When mm > 59 | mm < 00 | datatype(mm) <> 'NUM' then
    Call ERROR_MSG203
  When ss > 59 | ss < 00 | datatype(ss) <> 'NUM' then
    Call ERROR_MSG203
  When hh = 24 & mm > 00 then
    Call ERROR_MSG203
  Otherwise
    Badparm = ''
End

Return


/*-------------------------------------------------------------------*/
/* Get_Cname:                                                        */
/* Retrieve the Community Name from the TCP390 policy definition.    */
/* Also, retrieve the IPADDR & HOSTNAME keywords to determine        */
/* where to send the snmp request(s).                                */
/*-------------------------------------------------------------------*/
Get_Cname:

Arg Sp

If value(sp) = sp then
  SpVal = ''
Else
  SpVal = Value(sp)

cname = ''
sp_ipaddr = ''
sp_hname = ''
snmp_name = ''
snmpdata = ''

If SpVal = '' then                    /* No known sp data?           */
  Do                                  /* get cname, ipaddr, etc      */
    'PIPE (STAGESEP' sep') NETV ' ,
    'EZLEPOLY REQ=GET ENTRY=TCP390 TYPE='sp ,
       sep ' SEP ',
       sep ' LOC /COMMUNITYNAME / /IPADDR / /HOSTNAME / ',
       sep ' stem pipe.'

    /* TCP390 NMPIPL10 COMMUNITYNAME public */
    do j = 1 to pipe.0
      parse var pipe.j . . pipeparm pipedata
      select
        when pipeparm = 'COMMUNITYNAME' then
           cname = pipedata
        when pipeparm = 'IPADDR' then
           sp_ipaddr = pipedata
        when pipeparm = 'HOSTNAME' then
           sp_hname = pipedata
        otherwise
      end
    end

    /* set snmp hostname var to an ip addr                           */
    Select
      When sp_ipaddr <> '' then       /* got an ipaddr defined?      */
        snmp_name = sp_ipaddr
      When sp_hname <> '' then        /* hostname defined?           */
        Call Get_Host_By_Name         /* get its ipaddr then         */
      Otherwise                       /* oops - missing ipaddr/hostnm*/
        Call ERROR_MSG651
    End

    If cname = '' then                /* no community defined?       */
      cname = Default_Cname           /* Allow for a default         */

    /* Now, concatenate the community name & snmp name in            */
    snmpdata = "CNAME="cname" SNMPNAME="snmp_name
    spdata = spdata' 'snmpdata
    Interpret sp '= snmpdata'         /* save info under SP          */
  End                                 /* get cname, ipaddr, etc      */
Else                                  /* prior SP data saved         */
    spdata = spdata' 'Value(sp)


Return



/*-------------------------------------------------------------------*/
/* Get_IPPorts:                                                      */
/* Retrieve the IP port data from the Listener Table,                */
/* correlating the Local Port data with the Resource Name data.      */
/*                                                                   */
/* ibmMvsTcpListenerLocalPort MIB contains the port number:          */
/* CNM005I .1.3.6.1.4.1.2.6.19.2.2.7.31.1.4.16 = INTEGER: 23         */
/*                                                                   */
/* ibmMvsTcpListenerResourceName MIB contains the application name:  */
/* CNM005I .1.3.6.1.4.1.2.6.19.2.2.7.31.1.12.16 = STRING: TCPIP      */
/*                                                                   */
/* At the end of this subroutine the array, Port_Array, will contain */
/* the port number and application name pairs:                       */
/* Port_Array.nn = 23 TCPIP                                          */
/*-------------------------------------------------------------------*/
Get_IPPorts:

port_index.0 = 0

/* Initialize 1st line with a date and time stamp for debug.        */
port_cnt = 1
Port_Array.1 = "*** "Date('U')" "Time()
Port_Array.0 = port_cnt

ADDRESS NETVASIS

/* Build SNMP GETBULK command to retrieve LocalPort and             */
/* ResourceName MIB data from ibmMvsTcpListenerTable.               */
Cmd = 'snmp getbulk -v2c -Obestn -c 'cname' -Cr'MaxPorts ,
      ' -h ' snmp_name ,
      ibmMvsTcpListenerLocalPort ibmMvsTcpListenerResourceName

/* Issue SNMP command that was just built                           */
/* Need the LOC strings since -Cr100 may include unwanted MIBs.     */
'PIPE NETV ' cmd ,
  sep ' LOC /'ibmMvsTcpListenerLocalPort'/',
        ' /'ibmMvsTcpListenerResourceName'/',
  sep ' Stem MibResp.'

Do qq = 1 to MibResp.0 by 2               /* loop thru SNMP response */
/* *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  */
/* The GETBULK output will be pairs of MIBs with the first line      */
/* from the LocalPort MIB and the second line from the ResourceName  */
/* MIB similar to the following for TN3270 ports:                    */
/* CNM005I .1.3.6.1.4.1.2.6.19.2.2.7.31.1.4.17 = INTEGER: 23         */
/* CNM005I .1.3.6.1.4.1.2.6.19.2.2.7.31.1.12.17 = STRING: TCPIP      */
/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */
/* Parse above MIB response as follows:                              */
/* port_number = 23                                                  */
/* Appl_Name = TCPIP                                                 */
/* *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  */

 /* parse 1st line                                                   */
 MibData = MibResp.qq
 port_number = Word(MibData,Words(MibData))

 /* parse 2nd line                                                   */
 qq_next = qq + 1
 MibData = MibResp.qq_next
 Appl_Name = Word(MibData,Words(MibData))

 /* store port number and appl name in array                         */
 port_cnt = port_cnt + 1
 Port_Array.port_cnt = port_number' 'Appl_Name
End                                       /* loop thru SNMP response */

Port_Array.0 = port_cnt

Return


/*-------------------------------------------------------------------*/
/* Log_Ports:                                                        */
/* Log the port data (port number and application name) to           */
/* (1) File name stored in CNMSTYLE.CONNMONFILE                      */
/* (2) Safe name also known as CNMSTYLE.CONNMONFILE                  */
/*     Contents of SAFE can be used by calling routine when the      */
/*     collection type is DYNAMIC.  Caller can pull port data from   */
/*     safe to use for processing.                                   */
/*-------------------------------------------------------------------*/
Log_Ports:


'PIPE STEM Port_Array. ',              /* put port data to file       */
  sep ' INSTORE DSIPARM.'filename' COMMON REPLACE ',
  sep ' DROP FIRST ',                  /* drop time stamp line        */
  sep ' SAFE 'filename                 /* put port data in SAFE       */


If CollIntvl <> '' & BadParm = '' then  /* If a valid interval ...    */
  Do
   Interval = CollIntvl
   Call Reschedule_Timer                /* schedule port data coll    */
  End

Return



/*-------------------------------------------------------------------*/
/* Reschedule_Timer:                                                 */
/* Reschedule this Clist using the AFTER command.                    */
/*-------------------------------------------------------------------*/
Reschedule_Timer:

'EZLEADLY 'interval',ID='ident','ident Default_stack


Return


/*-------------------------------------------------------------------*/
/* Get_Host_By_Name:                                                 */
/* We were passed a hostname only.  We need to translate the         */
/* hostname into an IP addr.                                         */
/*-------------------------------------------------------------------*/
Get_Host_By_Name:

snmp_name = FKXEGHBN(sp_hname)

Return




/*-------------------------------------------------------------------*/
/* Error routines & Messages                                         */
/*-------------------------------------------------------------------*/



Error_Msg651:
/*-------------------------------------------------------------------*/
/* DSI651I KEYWORD MISSING - ONE OF FOLLOWING REQUIRED: keywords     */
/*-------------------------------------------------------------------*/
 EzlMsgs.0 = 1
 EzlMsgs.1 = ezlemsg('DSI651','N','IPADDR','HOSTNAME')
 Return_code = 3
 call LEAVE_NOW
Return



/*-------------------------------------------------------------------*/
/* EZL203I   EXPECTED PARAMETERS MISSING FOR REQUEST  ...            */
/*-------------------------------------------------------------------*/
ERROR_MSG203:

 EXITMSG = EZLEMSG(203,'N',Ident,Badparm)
 Return_Code = 4
 call Leave_now


/*-------------------------------------------------------------------*/
/* EZL204I   EXPECTED PARAMETERS INVALID FOR REQUEST ...             */
/*-------------------------------------------------------------------*/
ERROR_MSG204:

 EXITMSG = EZLEMSG(204,'N',Ident,Badparm,BadParmval)
 Return_Code = 4
 call Leave_now


/*-------------------------------------------------------------------*/
/* EZL205I   cmd  COMMAND FAILED ... - WAIT TIME EXPIRED             */
/*-------------------------------------------------------------------*/
ERROR_MSG205:

 EXITMSG = EZLEMSG(205,'N',Ident,Badcmd,Badcmd,Waittime)
 Return_Code = 5
 call Leave_now

/*-------------------------------------------------------------------*/
/* EZL206I   cmd COMMAND FAILED ... RECEIVED "rc"                    */
/* Errmsg                                                            */
/*-------------------------------------------------------------------*/
ERROR_MSG206:

 EXITMSG = EZLEMSG(206,'N',Ident,Badcmd,Badcmdrc,Errmsg)
 Return_Code = 5
 call Leave_now

/*-------------------------------------------------------------------*/
/* CLIST clist FAILED : LINE Sigl HAD A VARIABLE WITH NO VALUE       */
/*-------------------------------------------------------------------*/
NOVALUE:

 EXITMSG = EZLEMSG(271,'Y',ident,sigl,Condition('D'))
 Return_Code = 7                       /* return to caller           */

 call Leave_now

/*-------------------------------------------------------------------*/
/* Ident COMMAND FAILED FOR REXX : RECEIVED RETCODE=SYNTAX ERROR     */
/* BadRC on line Sigl                                                */
/*-------------------------------------------------------------------*/
SYNTAX:

 BadRC = RC
 EXITMSG = EZLEMSG(275,'Y',ident,sigl,BadRC)
 Say ExitMsg
 Return_code = 8

 call Leave_now

/*-------------------------------------------------------------------*/
/* GET OUT!!!!!                                                      */
/*-------------------------------------------------------------------*/
FAILURE:
   EZL_PFEXIT = 'PF2'
   'GLOBALV PUTT EZL_PFEXIT'
  'EZLTRACE EXIT 'ident '-1'
  Exit -1

HALT:
  EZL_PFEXIT = 'PF2'
  'GLOBALV PUTT EZL_PFEXIT'
  'EZLTRACE EXIT 'ident '-5'
 Exit -5


/*-------------------------------------------------------------------*/
/* Exit Trace Call                                                   */
/*-------------------------------------------------------------------*/
Leave_now:
 /* If Domain E/E trace is 'ON' or Oper E/E trace 'ON' ...           */
 If Substr(EZLTRACED,1,2) = 'ON' |,          /* If Domain on ..      */
    (Substr(EZLTRACED,1,3) = 'OFF' & OperTrace = 'ON') Then /*       */
   'EZLTRACE EXIT 'Ident Return_code         /* Call Exit Trace      */

   If EXITMSG <> '' then
     Say ExitMsg                                            /*  @P3A */
                                                            /* 2@P3D */

 Exit Return_code